package ru.mike.downloaderbot.processor.command;

import com.pengrad.telegrambot.TelegramBot;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.SendMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.mike.downloaderbot.domain.Rule;
import ru.mike.downloaderbot.service.RuleChannelService;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Component
public class RuleCommandProcessor implements CommandProcessor {
    @Autowired
    private TelegramBot telegramBot;

    @Autowired
    private RuleChannelService ruleChannelService;

    @Override
    public boolean isSuitableCommand(String command) {
        return command.startsWith("/rule");
    }

    @Override
    public void process(Update update) {
        Long currentChatId = update.message().chat().id();
        String text = update.message().text();

        String[] splitText = text.split(" ");

        String wrongRuleCommandFormatText = "Неверный формат команды для установки правила. " +
                "Формат команды:/rule c1 c2 c3\n" +
                "Где \n" +
                "c1 - имя канала, для которого применяется правило подписки" +
                "c2 - <имя канала, на который нужно подписаться>\n" +
                "c3 - <id канала, на который нужно подписаться>\n" +
                "c2 каналов может быть много, например \"/rule c1 c2 c2 c2";

        if (splitText.length <= 3) {
            sendCommandParseErrorMessage(currentChatId, text, wrongRuleCommandFormatText);
            return;
        }

        String ruleBaseChannelName = splitText[1];

        String[] ruleSubscribeChannelsNames = Arrays.copyOfRange(splitText, 2, splitText.length);

        if (ruleSubscribeChannelsNames.length % 2 != 0) {
            sendCommandParseErrorMessage(currentChatId, text, wrongRuleCommandFormatText);
            return;
        }

        List<String> channelsNames = new ArrayList<>();
        List<Long> channelsIds = new ArrayList<>();

        for (int i = 0; i < ruleSubscribeChannelsNames.length; i++) {
            String commandPart = ruleSubscribeChannelsNames[i];

            if (i % 2 == 0) {
                channelsNames.add(commandPart);
            } else {
                try {
                    channelsIds.add(Long.parseLong(commandPart));
                } catch (NumberFormatException e) {
                    sendCommandParseErrorMessage(currentChatId, text, wrongRuleCommandFormatText);
                    return;
                }
            }
        }

        Rule createdRule = ruleChannelService.createRuleAndChannels(ruleBaseChannelName, channelsNames, channelsIds);

        SendMessage successCreateRuleMessage = new SendMessage(currentChatId,
                createSuccessCreateRuleMessage(createdRule)
        );
        telegramBot.execute(successCreateRuleMessage);
    }

    private void sendCommandParseErrorMessage(Long currentChatId, String text, String wrongRuleCommandFormatText) {
        log.warn("Unable to parse /rule command:'{}'", text);

        SendMessage wrongFormatMessage = new SendMessage(currentChatId, wrongRuleCommandFormatText);
        telegramBot.execute(wrongFormatMessage);
    }

    private String createSuccessCreateRuleMessage(Rule createdRule) {
        String ruleChannelName = createdRule.getChannelName();
        String channelsStr = createdRule.getChannels().stream()
                .map(c -> "Имя: " + c.getName() + ", ссылка: " + c.getLink())
                .collect(Collectors.joining("\n"));

        return "Правило создано успешно\n" +
                "Имя канала правила:" + ruleChannelName +
                "\nКаналы для подписки:\n" +
                channelsStr;
    }
}
