package ru.mike.downloaderbot.processor.command;

import com.pengrad.telegrambot.TelegramBot;
import com.pengrad.telegrambot.model.ChatMember;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.request.GetChatMember;
import com.pengrad.telegrambot.request.SendMessage;
import com.pengrad.telegrambot.response.GetChatMemberResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.mike.downloaderbot.domain.Channel;
import ru.mike.downloaderbot.domain.LinkData;
import ru.mike.downloaderbot.service.LinkDataService;
import ru.mike.downloaderbot.service.RuleChannelService;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

@Slf4j
@Component
public class StartCommandProcessor implements CommandProcessor {
    @Autowired
    private TelegramBot telegramBot;

    @Autowired
    private LinkDataService linkDataService;

    @Autowired
    private RuleChannelService ruleChannelService;

    @Override
    public boolean isSuitableCommand(String command) {
        return command.startsWith("/start ");
    }

    @Override
    public void process(Update update) {
        Long currentChatId = update.message().chat().id();
        Long currentUserId = update.message().from().id();
        String text = update.message().text();

        String[] splitText = text.split(" ");

        if (splitText.length != 2) {
            log.warn("Unable to parse /start command:'{}'", text);

            SendMessage wrongFormatMessage = new SendMessage(currentChatId,
                    "Ой, что-то пошло не так! Попробуйте еще раз!");
            telegramBot.execute(wrongFormatMessage);
            return;
        }

        String linkIdStr = splitText[1];
        UUID linkId;

        try {
            linkId = UUID.fromString(linkIdStr);
        } catch (IllegalArgumentException e) {
            log.warn("Unable to parse link id. Command:'{}', id='{}'", text, linkIdStr);

            SendMessage wrongFormatMessage = new SendMessage(currentChatId,
                    "Ой, что-то пошло не так! Попробуйте еще раз!");
            telegramBot.execute(wrongFormatMessage);
            return;
        }

        Optional<LinkData> linkDataOptional = linkDataService.findById(linkId);

        if (linkDataOptional.isEmpty()) {
            SendMessage linkNotFoundMessage = new SendMessage(currentChatId,
                    "Не нашли ссылку на ресурс... Попробуйте другой!");
            telegramBot.execute(linkNotFoundMessage);
            return;
        }

        LinkData linkData = linkDataOptional.get();

        String channelName = linkData.getChannelName();

        List<Channel> channels = ruleChannelService.findChannelsByRuleChannelName(channelName);

        List<Long> channelsTgIds = channels.stream()
                .map(Channel::getTelegramChannelId)
                .collect(Collectors.toList());


        if (isAnyChannelNotSubscribed(currentUserId, channelsTgIds)) {
            String subscribeLinks = channels.stream()
                    .map(Channel::getLink)
                    .collect(Collectors.joining("\n"));

            String subscribeMessage = "Подпишись на каналы:\n" + subscribeLinks;

            telegramBot.execute(new SendMessage(currentChatId, subscribeMessage));
            return;
        }

        SendMessage linkMessage = new SendMessage(currentChatId,
                "Ссылка на ресурс: " + linkData.getResourceLink());
        telegramBot.execute(linkMessage);
    }

    private boolean isAnyChannelNotSubscribed(Long currentUserId, List<Long> channelsTgIds) {
        for (Long channelId : channelsTgIds) {
            GetChatMember chatMember = new GetChatMember(channelId, currentUserId);
            GetChatMemberResponse chatMemberResponse = telegramBot.execute(chatMember);

            if (Objects.isNull(chatMemberResponse) || Objects.isNull(chatMemberResponse.chatMember()) ||
                    chatMemberResponse.chatMember().status().equals(ChatMember.Status.left)) {
                return true;
            }
        }

        return false;
    }
}
